function [capital_pi small_pi p_T m]=tvc(y,X)

%TVC Regression with time varying coefficients.
%   GIVEN:
%   the linear model: y = X*beta+v
%   q=100 different degrees of coefficient instability
%   INPUTS:
%   X is a TxK matrix of regressors and
%   y is the Tx1 vector of dependent observations
%   OUTPUTS:
%   capital_pi and small_pi are the two measures of stability
%   p_T is the vector of the q posterior probabilities of the q degrees of instability
%   m is a TxKxq matrix of filtered coefficient estimates for each degree of instability (along the third dimension)   
%   OTHER INSTRUCTIONS:
%   use the slow version tvc_slow if this does not work
%   at the moment, only the slow version of this routine (tvc_slow) gives also smoothed coefficient estimates as output
%   for more detailed comments on the code see tvc slow


%-------- Find first non-zero observation to form the prior on variance

first_obs=0;
while first_obs==0
    first_obs=y(1);
    y=y(2:end);
    X=X(2:end,:);
end

%-------- Find sample size and number of observations

T=length(y);
K=size(X,2);

%-------- Compute omega

co=0;  %--- the variable called omega in the paper is called co here
MM=(X'*X)^-1;
co=sum(sum(X.*(X*MM)))/T;

%-------- Compute the grid for theta

hh=[0 fliplr(0.9.^(0:98))]*0.999;
h=hh./(co-hh*co);
NN=length(h);

%-------- Initialize the matrices that will hold all the variables

m=zeros(T,K,NN);
a=zeros(T,K,NN);
sm_a=zeros(T,K,NN);
C=zeros(K,K,T,NN);
C_=zeros(K,K,T,NN);
R=zeros(K,K,T,NN);
R_=zeros(K,K,T,NN);
sm_R_=zeros(K,K,T,NN);
f=zeros(T,1,NN);
Q=zeros(T,1,NN);
Q_=zeros(T,1,NN);
sm_B=zeros(K,K,T,NN);
A=zeros(T,K,NN);
e=zeros(T,1,NN);
n=zeros(T,1,NN);
S=zeros(T,1,NN);
W_=zeros(K,K,NN);
t_st=zeros(T,1,NN);
p_h=zeros(T,1,NN);

wo_g=(X'*X)^-1;
wo_K_K_1=zeros(K,K);
wo_K_K_2=zeros(K,K);

tvc_C(m,a,sm_a,C,C_,R,R_,sm_R_,f,Q,Q_,sm_B,A,e,n,S,W_,t_st,p_h,int32(NN),int32(T),int32(K),y,X,h,first_obs,wo_g,wo_K_K_1,wo_K_K_2);


%-------- Prepare output

p_T=squeeze(p_h(T,:,:));  %--- Posterior probabilities of the degrees of instability
modal=min(find(p_T==max(p_T)));  %--- Modal degree of instability
Htheta=find(p_T>p_T(1));  %--- Set Htheta in the paper
capital_pi=1-sum(p_T(Htheta))/sum(p_T(2:100));  %--- Capital pi in the paper
small_pi=p_T(1)/p_T(modal);  %--- Small pi in the paper
