function [capital_pi small_pi p_T m sm_a]=tvc_slow(y,X)

%TVC Regression with time varying coefficients.
%   GIVEN:
%   the linear model: y = X*beta+v
%   q=100 different degrees of coefficient instability
%   INPUTS:
%   X is a TxK matrix of regressors and
%   y is the Tx1 vector of dependent observations
%   OUTPUTS:
%   capital_pi and small_pi are the two measures of stability
%   p_T is the vector of the q posterior probabilities of the q degrees of instability
%   m is a TxKxq matrix of filtered coefficient estimates for each degree of instability (along the third dimension)   
%   sm_a is a TxKxq matrix of smoothed coefficient estimates for each degree of instability (along the third dimension)   
%   OTHER INSTRUCTIONS:
%   comments in the code match notation in the paper with notation in the code


%-------- Find first non-zero observation to form the prior on variance

first_obs=0;
while first_obs==0
    first_obs=y(1);
    y=y(2:end);
    X=X(2:end,:);
end

%-------- Find sample size and number of observations

T=length(y);
K=size(X,2);

%-------- Compute omega

co=0;  %--- the variable called omega in the paper is called co here
MM=(X'*X)^-1;
for j=1:T
   co=co+X(j,:)*MM*X(j,:)'; 
end
co=co/T;

%-------- Compute the grid for lambda

hh=[0 fliplr(0.9.^(0:98))]*0.999;  %--- lambda
h=hh./(co-hh*co);
NN=length(h);

%-------- Initialize the matrices that will hold all the variables

m=zeros(T,K,NN);
a=zeros(T,K,NN);
sm_a=zeros(T,K,NN);
C=zeros(K,K,T,NN);
C_=zeros(K,K,T,NN);
R=zeros(K,K,T,NN);
R_=zeros(K,K,T,NN);
sm_R_=zeros(K,K,T,NN);
f=zeros(T,1,NN);
Q=zeros(T,1,NN);
Q_=zeros(T,1,NN);
sm_B=zeros(K,K,T,NN);
A=zeros(T,K,NN);
e=zeros(T,1,NN);
n=zeros(T,1,NN);
S=zeros(T,1,NN);
W_=zeros(K,K,NN);
t_st=zeros(T,1,NN);
p_h=zeros(T,1,NN);

%-------- Initialize all the variables

for nn=1:NN  %--- Loop over the grid (degrees of instability)

a(1,:,nn)=zeros(1,K);  %--- beta_(t)_(t-1)
R_(:,:,1,nn)=T*(X'*X)^-1;  %--- F_beta_(t)_(t-1)
W_(:,:,nn)=h(nn)*(X'*X)^-1;   %--- lambda*F_beta_(1)_(0)
f(1,:,nn)=X(1,:)*a(1,:,nn)';  %--- y_(t)_(t-1)
Q_(1,:,nn)=1+X(1,:)*R_(:,:,1,nn)*X(1,:)';  %--- F_y_(t)_(t-1)
e(1,:,nn)=y(1)-f(1,:,nn);  %--- e_(t)
A(1,:,nn)=(R_(:,:,1,nn)*X(1,:)')'./Q_(1,:,nn);  %--- P_(t)
m(1,:,nn)=a(1,:,nn)+A(1,:,nn).*e(1,:,nn);  %--- beta_(t)_(t)
C_(:,:,1,nn)=R_(:,:,1,nn)-A(1,:,nn)'*A(1,:,nn).*Q_(1,:,nn);  %--- F_beta_(t)_(t)
n(1,:,nn)=1+1;  %--- n_(t)
S(1,:,nn)=(first_obs^2+(e(1,:,nn)^2)/Q_(1,:,nn))/n(1,:,nn);  %--- V_(t)
R(:,:,1,nn)=first_obs^2*R_(:,:,1,nn);  %--- V_(t-1)*F_beta_(t)_(t-1)
Q(1,:,nn)=first_obs^2*Q_(1,:,nn);  %--- V_(t-1)*F_y_(t)_(t-1)
C(:,:,1,nn)=S(1,:,nn)*C_(:,:,1,nn);  %--- V_(t)*F_beta_(t)_(t)

for t=2:T  %-- Forward Kalman recursions
    
a(t,:,nn)=m(t-1,:,nn);  %--- beta_(t)_(t-1)
R_(:,:,t,nn)=C_(:,:,t-1,nn)+W_(:,:,nn);  %--- F_beta_(t)_(t-1)
f(t,:,nn)=X(t,:)*a(t,:,nn)';  %--- y_(t)_(t-1)
Q_(t,:,nn)=1+X(t,:)*R_(:,:,t,nn)*X(t,:)';  %--- F_y_(t)_(t-1)
e(t,:,nn)=y(t)-f(t,:,nn);  %--- e_(t)
A(t,:,nn)=(R_(:,:,t,nn)*X(t,:)')'./Q_(t,:,nn);  %--- P_(t)
m(t,:,nn)=a(t,:,nn)+A(t,:,nn).*e(t,:,nn);  %--- beta_(t)_(t)
C_(:,:,t,nn)=R_(:,:,t,nn)-A(t,:,nn)'*A(t,:,nn).*Q_(t,:,nn);  %--- F_beta_(t)_(t)
n(t,:,nn)=n(t-1,:,nn)+1;  %--- n_(t)
S(t,:,nn)=(n(t-1,:,nn)*S(t-1,:,nn)+(e(t,:,nn)^2)/Q_(t,:,nn))/n(t,:,nn);  %--- V_(t)
R(:,:,t,nn)=S(t-1,:,nn)*R_(:,:,t,nn);  %--- V_(t-1)*F_beta_(t)_(t-1)
Q(t,:,nn)=S(t-1,:,nn)*Q_(t,:,nn);  %--- V_(t-1)*F_y_(t)_(t-1)
C(:,:,t,nn)=S(t,:,nn)*C_(:,:,t,nn);  %--- V_(t)*F_beta_(t)_(t) 
    
end

sm_a(T,:,nn)=m(T,:,nn);
sm_R_(:,:,T,nn)=C_(:,:,T,nn);

for t=1:(T-1)  %-- Backward Kalman recursions
    
   sm_B(:,:,T-t,nn)=C_(:,:,T-t,nn)*R_(:,:,T-t+1,nn)^-1;
   sm_a(T-t,:,nn)=m(T-t,:,nn)+(sm_B(:,:,T-t,nn)*(sm_a(T-t+1,:,nn)-a(T-t+1,:,nn))')';  %--- beta_(T-t)_(T)
   sm_R_(:,:,T-t,nn)=C_(:,:,T-t,nn)+sm_B(:,:,T-t,nn)*(sm_R_(:,:,T-t+1,nn)-R_(:,:,T-t+1,nn))*sm_B(:,:,T-t,nn)';  %--- F_beta_(T-t)_(T)
    
end

end

%-------- Compute conditional likelihood and posterior probability for each degree of instability

for nn=1:NN
t_st(1,:,nn)=(1+(e(1,:,nn)^2)/(Q(1,:,nn)*1))^(-0.5*(1+1));
t_st(1,:,nn)=t_st(1,:,nn)/sqrt(Q(1,:,nn));
end
p_h(1,:,:)=t_st(1,:,:);
p_h(1,:,:)=p_h(1,:,:)/sum(p_h(1,:,:));

for t=2:T
   for nn=1:NN
    t_st(t,:,nn)=(1+(e(t,:,nn)^2)/(Q(t,:,nn)*n(t-1,:,nn)))^(-0.5*(n(t-1,:,nn)+1));
    t_st(t,:,nn)=t_st(t,:,nn)/sqrt(Q(t,:,nn));
   end
   p_h(t,:,:)=t_st(t,:,:).*p_h(t-1,:,:);
   p_h(t,:,:)=p_h(t,:,:)/sum(p_h(t,:,:));
end

%-------- Prepare output

p_T=squeeze(p_h(T,:,:));  %--- Posterior probabilities of the degrees of instability
modal=min(find(p_T==max(p_T)));  %--- Modal degree of instability
Htheta=find(p_T>p_T(1));  %--- Set Htheta in the paper
capital_pi=1-sum(p_T(Htheta))/sum(p_T(2:100));  %--- Capital pi in the paper
small_pi=p_T(1)/p_T(modal);  %--- Small pi in the paper
